{% extends "common/step.jinja" %}
{% block step_content %}

    <h3 class="fs-5">PCA data</h3>
    <h3 class="fs-6">Data snapshot</h3>
    <div class="table-responsive overflow-auto mb-4" style="max-height: 400px;">
        {{ data[i].output["pca_data"] }}
    </div>

    <h3 class="fs-5">Explained variance plot</h3>
    {{ data[i].output["explained_variance"] }}
    {{ data[i].output["pca_plot"] }}
{% endblock %}